@@ -9,7 +9,8 @@ from itertools import product
 import pickle
 import h5py as hdf
 import datetime, shutil
-import DMCpy.DataFile
+from DMCpy.FileStructure import shallowRead, HDFTranslationAlternatives, HDFTranslation, HDFCounts
+
 import DMCpy
 
 
@@ -229,19 +230,31 @@ def binData3D(dx,dy,dz,pos,data,norm=None,mon=None,bins=None):
 
     #pos = [np.array(x[NonNaNs]) for x in pos]
     HistBins = [bins[0][:,0,0],bins[1][0,:,0],bins[2][0,0,:]]
-    intensity =    np.histogramdd(np.array(pos).T,bins=HistBins,weights=data.flatten())[0].astype(data.dtype)
 
-    returndata = [intensity]
-    if mon is not None:
-        MonitorCount=  np.histogramdd(np.array(pos).T,bins=HistBins,weights=mon.flatten())[0].astype(mon.dtype)
-        returndata.append(MonitorCount)
-    if norm is not None:
-        Normalization= np.histogramdd(np.array(pos).T,bins=HistBins,weights=norm.flatten())[0].astype(norm.dtype)
+    if False:
+        intensity =    np.histogramdd(np.array(pos).T,bins=HistBins,weights=data.flatten())[0].astype(data.dtype)
+
+        returndata = [intensity]
+        if mon is not None:
+            MonitorCount=  np.histogramdd(np.array(pos).T,bins=HistBins,weights=mon.flatten())[0].astype(mon.dtype)
+            returndata.append(MonitorCount)
+        if norm is not None:
+            Normalization= np.histogramdd(np.array(pos).T,bins=HistBins,weights=norm.flatten())[0].astype(norm.dtype)
+            
+            returndata.append(Normalization)
+            
+        NormCount =    np.histogramdd(np.array(pos).T,bins=HistBins,weights=np.ones_like(data).flatten())[0].astype(float)
+        returndata.append(NormCount)
+
+    else:
+        weights = [data.flatten()]
+        if not mon is None:
+            weights.append(mon.flatten())
+        if not norm is None:
+            weights.append(norm.flatten())
         
-        returndata.append(Normalization)
-        
-    NormCount =    np.histogramdd(np.array(pos).T,bins=HistBins,weights=np.ones_like(data).flatten())[0].astype(float)
-    returndata.append(NormCount)
+        returndata = histogramdd(np.array(pos).T,bins=HistBins,weights=weights,returnCounts=True)
+
     return returndata,bins
 
 
@@ -846,7 +859,7 @@ def merge(dataFilesList,saveFileName,directory=None, A3Tolerance=0.05, A4Toleran
     # Perform checks
     equalParameters = ['twoThetaPosition','wavelength']
     equalParametersTolerance = [A4Tolerance,wavelengthTolerance]
-    files = DMCpy.DataFile.shallowRead(dataFilesList,equalParameters)
+    files = shallowRead(dataFilesList,equalParameters)
     
     trueValue = None
     truthTable = []
@@ -883,7 +896,7 @@ def merge(dataFilesList,saveFileName,directory=None, A3Tolerance=0.05, A4Toleran
     shutil.copy(dataFilesList[0],savepath)
     
     # Find min and max of A3 as well as average across all files
-    A3files = DMCpy.DataFile.shallowRead(dataFilesList,['A3'])
+    A3files = shallowRead(dataFilesList,['A3'])
     
     # Check if length is 1, then it is a powder!!!
     powderFiles = [f['file'] for f in A3files if len(f['A3'])<2 ]
@@ -918,10 +931,10 @@ def merge(dataFilesList,saveFileName,directory=None, A3Tolerance=0.05, A4Toleran
     
     def getPositionInFile(file,parameter):
         """ Get position of specified parameter in HDF file"""
-        if not parameter in DMCpy.DataFile.HDFTranslationAlternatives:
-                intensityPositionsInFile = [DMCpy.DataFile.HDFTranslation[parameter]]
+        if not parameter in HDFTranslationAlternatives:
+                intensityPositionsInFile = [HDFTranslation[parameter]]
         else:
-            intensityPositionsInFile = DMCpy.DataFile.HDFTranslationAlternatives[parameter]
+            intensityPositionsInFile = HDFTranslationAlternatives[parameter]
             
         if len(intensityPositionsInFile)>1: # If there are multiple possible positions
             for pos in intensityPositionsInFile[::-1]: # Last entry is the newest
@@ -949,7 +962,7 @@ def merge(dataFilesList,saveFileName,directory=None, A3Tolerance=0.05, A4Toleran
     totalSteps = len(newA3)
     with hdf.File(savepath,'r') as saveFile:
         # Find positions in dataFilesList
-        countPositionInFile = '/'+DMCpy.DataFile.HDFCounts#
+        countPositionInFile = '/'+HDFCounts#
         countShape = saveFile[countPositionInFile].shape
         monitorPositionInFile = getPositionInFile(saveFile,'monitor')
         summedCountsPositionInFile = getPositionInFile(saveFile,'summedCounts')
@@ -1021,3 +1034,103 @@ def merge(dataFilesList,saveFileName,directory=None, A3Tolerance=0.05, A4Toleran
         proc.create_dataset('rawdata',shape=(len(dataFilesList),),data=np.asarray(rawData,dtype='S'))
         
         print('Data merged and saved in {:}'.format(savepath))
+
+
+def histogramdd(sample, bins, weights, returnCounts = False):
+    """
+    Restricted version of numpys multidimensional histogram function. 
+
+    Args:
+
+        - sample (n x m array): Position in m-dimensional space
+
+        - bins (m list): List of bins
+
+        - weights (list): List of weights where each entry has the length n
+
+    Kwargs:
+
+        - returnCounts (bool): if True return also number of entries in each bin (default False)
+
+    """
+
+    try:
+        # Sample is an ND-array.
+        N, D = sample.shape
+    except (AttributeError, ValueError):
+        # Sample is a sequence of 1D arrays.
+        sample = np.atleast_2d(sample).T
+        N, D = sample.shape
+
+
+    try:
+        M = len(bins)
+        if M != D:
+            raise ValueError(
+                'The dimension of bins must be equal to the dimension of the '
+                ' sample x.')
+    except TypeError:
+        # bins is an integer
+        bins = D*[bins]
+
+
+    nbin = np.empty(D, int)
+    edges = D*[None]
+    for i in range(D):
+        edges[i] = np.asarray(bins[i])
+        nbin[i] = len(edges[i])+1
+    
+
+    # Compute the bin number each sample falls into.
+    Ncount = tuple(
+        # avoid np.digitize to work around gh-11022
+        np.searchsorted(edges[i], sample[:, i], side='right')
+        for i in range(D)
+    )
+
+    # Using digitize, values that fall on an edge are put in the right bin.
+    # For the rightmost bin, we want values equal to the right edge to be
+    # counted in the last bin, and not as an outlier.
+    for i in range(D):
+        # Find which points are on the rightmost edge.
+        on_edge = (sample[:, i] == edges[i][-1])
+        # Shift these points one bin to the left.
+        Ncount[i][on_edge] -= 1
+
+    # Compute the sample indices in the flattened histogram matrix.
+    # This raises an error if the array is too large.
+    xy = np.ravel_multi_index(Ncount, nbin)
+
+    # Compute the number of repetitions in xy and assign it to the
+    # flattened histmat.
+
+    histograms = []
+    for w in weights:
+        hist = np.bincount(xy, w, minlength=nbin.prod())
+
+        # Shape into a proper matrix
+        hist = hist.reshape(nbin)
+
+        # This preserves the (bad) behavior observed in gh-7845, for now.
+        hist = hist.astype(w.dtype)#, casting='safe')
+
+        # Remove outliers (indices 0 and -1 for each dimension).
+        core = D*(slice(1, -1),)
+        hist = hist[core]
+        histograms.append(hist)
+
+    if returnCounts:
+        hist = np.bincount(xy, minlength=nbin.prod())
+
+        # Shape into a proper matrix
+        hist = hist.reshape(nbin)
+
+        # This preserves the (bad) behavior observed in gh-7845, for now.
+        hist = hist.astype(int)#, casting='safe')
+
+        # Remove outliers (indices 0 and -1 for each dimension).
+        core = D*(slice(1, -1),)
+        hist = hist[core]
+        histograms.append(hist)
+
+    return histograms
\ No newline at end of file
